;		Siteplayer Servo Interface
;		X/Y input range 0-125
;		Center on power up
;		Controls two servos from IP input
;		X Axis reversed
;		4 MHz clock
;		4/21/04			
;------------------------------------------------------------
        LIST    P=16F627
	ERRORLEVEL	-302
        include <P16F627.INC>
        __CONFIG        h'3d01'
;-------------------------------------------------------------
;               Port Set-up
;           0   1   2   3   4   5   6   7       bit
; port  A   Out Out Out Out Out Out X   X
;       B   In  In  In  Out Out Out Out Out
;------------------------------------------------------------
	ORG	0			; start code
	goto	main
	ORG	4
	goto	int_service
;------------------------------------------------------------
        cblock  0x20
                temp_S                  ; 20
                temp_W                  ; 21 int regs
                FLAGS                   ; 22 various flag bits
                serial_reg              ; 23 serial data
		target_x		; 24
		target_y		; 25
		pulse_x			; 26	
		pulse_y			; 27
        endc
;------------------------------------------------------------
#define	X_AXIS		PORTA,0		;x servo pulse out
#define	Y_AXIS		PORTA,1		;y servo pulse out
#define	RX_DATA		FLAGS,0		;async byte rx'd
;
PULSE_GAP	equ	d'20000'	;20 mS
PULSE_MINIMUM	equ	d'125'		;1 ms / 8 uS
PULSE_CENTERED	equ	d'187'		;1.5 mS
;------------------------------------------------------------
main
        clrf    PORTA
        clrf    PORTB
        movlw   0x07
        movwf   CMCON                   ;compare module off
        bsf     STATUS,RP0              ;switch to bank 1
        movlw   b'00000000'             ;port A in/outputs
        movwf   TRISA
        movlw   b'00000111'             ;port B in/outputs
        movwf   TRISB
        movlw   b'11000010'		;int on rise
        movwf   OPTION_REG              ;TMR0/8 
	movlw	b'00000001'
	movwf	PIE1  			;TMR1 int enabled
        movlw   b'00100100'
        movwf   TXSTA                   ;tx ON, BRGH hi
	movlw	d'25'			;9k6
        movwf   SPBRG                   ;set data speed
        bcf     STATUS,RP0              ;bank 0
;
        movlw   b'10010000'             ;serial enabled
        movwf   RCSTA                   ;serial rx prep
        clrf    PIR1                    ;reset perif int
	movlw	b'00000001'		;T1/1, on
	movwf	T1CON			;TMR1 setup
	movlw	b'01000000'
        movwf   INTCON                  ;PIE enabled
;------------------------------------------------------------
start_here
	movlw	PULSE_CENTERED		;1.5 mS pulse
	movwf	pulse_x
	movwf	pulse_y
	movlw	PULSE_CENTERED - PULSE_MINIMUM	;
	movwf	target_x
	movwf	target_y
;
	movlw	high PULSE_GAP 
	subwf	TMR1H,F
	movlw	low PULSE_GAP 		;reload timeout
	subwf	TMR1L,F
	bcf	PIR1,TMR1IF		;clear flag
	bsf	INTCON,GIE		;global ints on
;
input_data_loop
	btfss	INTCON,INTF		;low pulse on B:0 ?
	goto	$-1
	bcf	INTCON,INTF		;yes, get x,y position	
query_SP
	movlw	0
	call	serial_tx		;send NOP
	movlw	0xc1
	call	serial_tx		;read 2 bytes
	movlw	d'10'
	call serial_tx			;from mem loc 10 dec
;	
rx_SP_data
	btfss   PIR1,RCIF
	goto	$-1
	call	serial_rx_handler
	btfss	RX_DATA
	goto	input_data_loop		;rx data error
	movlw	d'126'
	subwf	serial_reg,W
	bc	$+4			;> 125, don't use
	movfw	serial_reg
	sublw	d'125'			;reverse servo
	movwf	target_x		;rx data to target_x
;
	btfss   PIR1,RCIF
	goto	$-1
	call	serial_rx_handler
	btfss	RX_DATA
	goto	input_data_loop		;rx data error
	movlw	d'126'
	subwf	serial_reg,W
	bc	$+3			;> 125, don't use
	movfw	serial_reg
	movwf	target_y		;rx data to target_y
	goto	input_data_loop		
;
;------------------------------------------------------------
int_service
        movwf   temp_W			;store W and STATUS regs
        swapf   STATUS,W
        movwf   temp_S
	bcf	STATUS,RP0		;bank 0
;
	btfsc	PIR1,TMR1IF		;test for TMR1 interrupt
	call	T1_handler		;flag set, call routine
;
outta_here
        swapf   temp_S,W		;restore W and STATUS regs
        movwf   STATUS
        swapf   temp_W,F
        swapf   temp_W,W
        retfie                          ;return to program
;------------------------------------------------------------
;		Subroutines
;------------------------------------------------------------
T1_handler
	bcf	PIR1,TMR1IF		;clear flag
	movlw	high PULSE_GAP 
	subwf	TMR1H,F
	movlw	low PULSE_GAP 		;reload timeout
	subwf	TMR1L,F
;
pw_calc_x
	movlw	PULSE_MINIMUM		;
	addwf	target_x,W		;1 mS + 0-1 mS
	subwf	pulse_x,W
	bz	pw_calc_y		;target = pulse
	bnc	$+3			;target_x > pulse_x
	decf	pulse_x,F		;pulse_x -1
	goto	pw_calc_y
	incf	pulse_x,F		;pulse_x +1
;
pw_calc_y
	movlw	PULSE_MINIMUM
	addwf	target_y,W		;1 mS + 0-1 mS
	subwf	pulse_y,W
	bz	set_pulse_x		;target = pulse
	bnc	$+3			;target_y > pulse_y
	decf	pulse_y,F		;pulse_y -1
	goto	set_pulse_x
	incf	pulse_y,F		;pulse_y +1
;
set_pulse_x
	clrf	TMR0
	movfw	pulse_x
	subwf	TMR0,F
	bcf	INTCON,T0IF
	bsf	X_AXIS
	btfss	INTCON,T0IF
	goto	$-1
	bcf	X_AXIS
;
set_pulse_y
	movfw	pulse_y
	subwf	TMR0,F
	bcf	INTCON,T0IF
	bsf	Y_AXIS
	btfss	INTCON,T0IF
	goto	$-1
	bcf	Y_AXIS
	return
;------------------------------------------------------------
serial_rx_handler
	bcf	RX_DATA			;clear flag
        movlw   b'00000110'
        andwf   RCSTA,W                 ;check error flags
        bnz     serial_error
        movfw   RCREG
        movwf   serial_reg              ;xfer data byte
	bsf	RX_DATA			;yes, byte rx'd
        return
serial_error
        btfss   RCSTA,FERR
        goto    check_orrun
        movfw   RCREG                   ;read byte, don't store
check_orrun
        btfss   RCSTA,OERR
        return
        bcf     RCSTA,CREN
        nop
        bsf     RCSTA,CREN              ;cycle CREN bit to reset
        return
;------------------------------------------------------------
serial_tx
        btfss   PIR1,TXIF		;test if ready for new byte
        goto    $-1
	nop
        movwf   TXREG                   ;load W to send
	return
;------------------------------------------------------------
;------------------------------------------------------------
        end
;------------------------------------------------------------
Copyright (c) 2004, Mike Berg
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions 
are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer. 
Redistributions in binary form must reproduce the above copyright 
notice, this list of conditions and the following disclaimer in the 
documentation and/or other materials provided with the distribution. 
Neither the name of the <ORGANIZATION> nor the names of its 
contributors may be used to endorse or promote products derived from 
this software without specific prior written permission. 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
